#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Modelos del modulo :Resumen mensual de nomina:"""
from osv import fields, osv
from tools.translate import _
from datetime import datetime
import xlwt


class FitCol(object):
    """Ajusta el ancho de las columnas automaticamente"""

    def __init__(self, sheet):
        self.sheet = sheet
        self.widths = dict()

    def write(self, r, c, label='', *args, **kwargs):
        if isinstance(label, str) and len(args) == 0:
            data = label.decode('utf-8')
        elif isinstance(label, str) and len(args) > 0:
            data = datetime.strptime(label, '%Y-%m-%d')
        else:
            data = label
        self.sheet.write(r, c, data, *args, **kwargs)
        if isinstance(label, (int, float)):
            label = str(label).decode('utf-8')
        width = (len(label) + 1) * 256
        if width > self.widths.get(c, 0):
            self.widths[c] = width
            self.sheet.col(c).width = width

    def __getattr__(self, attr):
        return getattr(self.sheet, attr)


class sisb_nomina_resumen(osv.osv_memory):
    """Asistente para el reporte del resumen de nomina"""
    _name = 'sisb.nomina.resumen'
    _columns = {
        'month': fields.many2one('hr.month', 'Month', required=True)
    }

    def get_employees_data(self, cr, month):
        """ Query a Postgres para obtener informacion de los empleados en un mes.

            :param month: Nombre del mes a consultar.
        """
        query = '''
            SELECT
                hde.payroll as nom,
                he.name as emp,
                he.identification_id as ide,
                date_start_rel_laboral(he.id)  as dst,
                hde.department_id as dpt,
                hde.job as job,
                case when hpl.type_calc = 'deduction' then 'debt' else 'bonus' end as typ,
                hc.name as con,
                sum(hpl.total)
            FROM
                hr_payslip_line as hpl
                join hr_payslip as hrp on hrp.id = hpl.slip_id
                join hr_employee as he on he.id = hpl.employee_id
                join hr_department_employee as hde on hde.employee_id = he.id
                    and hde.date_start < hrp.date_from and (hde.date_end > hrp.date_from or hde.date_end is null)
                join hr_concept as hc on hc.id = hpl.concept_id and hc.type <> 'aporte_patronal'
                join hr_process as hp on hp.id = hpl.process_id and hp.type in ('nomina')
                join hr_period_process_rel as hppr on hppr.process_id = hp.id
                join hr_period as hpe on hpe.id = hppr.period_id
                join hr_month as hm on hm.id = hpe.hr_month_id and hm.name = '%s'
            WHERE appears_on_payslip = True
            group by emp, con, typ, ide, nom, dst, dpt, job
            order by emp, typ, con
        ''' % (month)

        cr.execute(query)
        employees = {}
        for roster, name, identification, join_date, department, job, asign, concept, total in cr.fetchall():
            if identification not in employees:
                employees[identification] = {}
            employees[identification]['roster'] = roster
            employees[identification]['name'] = name
            employees[identification]['identification'] = identification
            employees[identification]['join_date'] = join_date
            employees[identification]['department'] = department
            employees[identification]['job'] = job
            if 'bonuses' not in employees[identification]:
                employees[identification]['bonuses'] = {}
            if 'debts' not in employees[identification]:
                employees[identification]['debts'] = {}
            if asign == 'bonus':
                employees[identification]['bonuses'][concept] = total
            else:
                employees[identification]['debts'][concept] = total

        return employees

    def get_concepts_header(self, cr, month):
        """ Query que se trae todos los conceptos de asignaciones y deducciones de los trabajadores en el mes indicado.

            :param month: Nombre del mes a consultar.
        """
        query = '''
        SELECT
            case when hpl.type_calc = 'deduction' then 'debt' else 'bonus' end as asigned,
            hc.name as concept,
            hc.code as code
        FROM
            hr_payslip_line as hpl
            join hr_employee as he on he.id = hpl.employee_id
            join hr_payroll as hno on hno.id = he.hr_payroll_id
            join hr_concept as hc on hc.id = hpl.concept_id and hc.type <> 'aporte_patronal'
            join hr_process as hp on hp.id = hpl.process_id and hp.type in ('nomina')
            join hr_period_process_rel as hppr on hppr.process_id = hp.id
            join hr_period as hpe on hpe.id = hppr.period_id
            join hr_month as hm on hm.id = hpe.hr_month_id and hm.name = '%s'
        WHERE appears_on_payslip = True
        group by concept, asigned, hc.code
        order by asigned, concept
        ''' % (month)

        cr.execute(query)
        bonuses = ['']
        debts = []
        for row in cr.fetchall():
            con_type, concept, code = row
            if con_type == 'bonus':
                if code == '0100':
                    bonuses[0] = concept
                else:
                    bonuses.append(concept)
            else:
                debts.append(concept)

        return {'bonuses': bonuses, 'debts': debts}

    def get_payrolls_pages(self, cr, month):
        """ Query que se trae los nombres de las nominas de los trabajadores del mes indicado.

            :param month: Nombre del mes a consultar.
        """
        query = '''
        SELECT
            hde.payroll as roster
        FROM
            hr_payslip_line as hpl
            join hr_payslip as hrp on hrp.id = hpl.slip_id
            join hr_employee as he on he.id = hpl.employee_id
            join hr_payroll as hno on hno.id = he.hr_payroll_id
            join hr_department_employee as hde on hde.employee_id = he.id and hde.date_start < hrp.date_from
                and (hde.date_end > hrp.date_from or hde.date_end is null)
            join hr_process as hp on hp.id = hpl.process_id and hp.type in ('nomina')
            join hr_period_process_rel as hppr on hppr.process_id = hp.id
            join hr_period as hpe on hpe.id = hppr.period_id
            join hr_month as hm on hm.id = hpe.hr_month_id and hm.name = '%s'
        WHERE appears_on_payslip = True
        group by roster
        ''' % (month)

        cr.execute(query)
        return [row[0] for row in cr.fetchall()]

    def print_results(self, cr, uid, ids, context=None):
        """ Funcion llamada por el wizard para imprimir los resultados.
        """
        month = self.browse(cr, uid, ids)[0].month.name
        rosters = self.get_payrolls_pages(cr, month)
        doc = xlwt.Workbook()
        pages = {}
        j = {}  # Seran los indices de las filas de cada pagina
        for roster in rosters:
            roster = roster[:31]  # Por limitaciones de los archivos xls, los nombres de paginas no pueden exceder los 31 caracteres
            # Se crean las paginas y se guardan. Se le van creando las cabeceras
            pages[roster] = doc.add_sheet(roster, True)
            pages[roster].protect = True
            pages[roster] = FitCol(pages[roster])
            pages[roster].write(0, 0, _('N°'))
            pages[roster].write(0, 1, _('Employee'))
            pages[roster].write(0, 2, _('Identification'))
            pages[roster].write(0, 3, _('Join date'))
            pages[roster].write(0, 4, _('Department'))
            pages[roster].write(0, 5, _('Job'))
            j[roster] = 1

        # Se procede a crear las cabeceras de conceptos en cada una de las paginas
        i = 6  # Columns index
        columns = {}
        concepts = self.get_concepts_header(cr, month)
        bonuses = concepts['bonuses']
        debts = concepts['debts']
        for bonus in bonuses:
            for page in pages.values():
                page.write(0, i, bonus)
            columns[bonus] = i
            i += 1
        for page in pages.values():
            page.write(0, i, _('Total bonuses'))
        columns['bonuses'] = i
        i += 1
        for debt in debts:
            for page in pages.values():
                page.write(0, i, debt)
            columns[debt] = i
            i += 1
        for page in pages.values():
            page.write(0, i, _('Total debts'))
        columns['debts'] = i
        i += 1
        for page in pages.values():
            page.write(0, i, _('Net Total'))
        columns['net_total'] = i

        # -------- EMPEZAMOS A LLENAR LA DATA DE LOS TRABAJADORES ------------
        datas = self.get_employees_data(cr, month)
        date_format = xlwt.XFStyle()
        date_format.num_format_str = 'dd/mm/yyyy'
        currency_format = xlwt.XFStyle()
        currency_format.num_format_str = "#,##0.00"
        for data in datas:
            n = datas[data]['roster'][:31]
            pages[n].write(j[n], 0, j[n])
            pages[n].write(j[n], 1, datas[data]['name'])
            pages[n].write(j[n], 2, int(datas[data]['identification']))
            pages[n].write(j[n], 3, datas[data]['join_date'], date_format)
            pages[n].write(j[n], 4, datas[data]['department'])
            pages[n].write(j[n], 5, datas[data]['job'])
            total_bonuses = 0
            for bonus in datas[data]['bonuses']:
                pages[n].write(j[n], columns[bonus], datas[data]['bonuses'][bonus], currency_format)
                total_bonuses += datas[data]['bonuses'][bonus]
            pages[n].write(j[n], columns['bonuses'], total_bonuses, currency_format)
            total_debts = 0
            for debt in datas[data]['debts']:
                pages[n].write(j[n], columns[debt], datas[data]['debts'][debt], currency_format)
                total_debts += datas[data]['debts'][debt]
            pages[n].write(j[n], columns['debts'], total_debts, currency_format)
            pages[n].write(j[n], columns['net_total'], total_bonuses - total_debts, currency_format)
            j[n] += 1
        try:
            file_name = '/tmp/' + _('roster_report_') + datetime.now().strftime('%H%M%S%d%m%Y')
            doc.save(file_name)
        except:
            raise osv.except_osv('ERROR', _('No data found'))

        data = {}
        data['file'] = file_name
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'nomina.resumen',
            'datas': data,
        }


sisb_nomina_resumen()
